package com.hzya.frame.plugin.lets.plugin.outsourc;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.lang.Assert;
import cn.hutool.core.util.StrUtil;
import cn.hutool.http.HttpRequest;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.hzya.frame.base.PluginBaseEntity;
import com.hzya.frame.beanutil.BeanUtil;
import com.hzya.frame.plugin.lets.constant.OverallConstant;
import com.hzya.frame.plugin.lets.constant.ProfilesActiveConstant;
import com.hzya.frame.plugin.lets.dao.*;
import com.hzya.frame.plugin.lets.entity.*;
import com.hzya.frame.plugin.lets.ofsvo.QueryOfsSoSaleOutVo;
import com.hzya.frame.plugin.lets.queryvo.StartAndEndVo;
import com.hzya.frame.plugin.lets.u8cdto.*;
import com.hzya.frame.plugin.lets.util.*;
import com.hzya.frame.split.SplitListByCountUtil;
import com.hzya.frame.sysnew.integtationTaskLivingDetails.dao.IIntegrationTaskLivingDetailsDao;
import com.hzya.frame.sysnew.integtationTaskLivingDetails.entity.IntegrationTaskLivingDetailsEntity;
import com.hzya.frame.ttxofs.dto.ofsReceiptConfirmSearch.OFSReceiptConfirmSearchResponse;
import com.hzya.frame.ttxofs.dto.ofspoorder.OfsPoOrderData;
import com.hzya.frame.ttxofs.dto.ofspoorder.OfsPoOrderDetails;
import com.hzya.frame.ttxofs.dto.ofspoorder.OfsPoOrderHeader;
import com.hzya.frame.ttxofs.dto.stock.StockinOrderSearchResponse;
import com.hzya.frame.web.entity.BaseResult;
import com.hzya.frame.web.entity.JsonResultEntity;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;

import java.math.BigDecimal;
import java.util.*;
import java.util.concurrent.locks.ReentrantLock;
import java.util.function.Predicate;
import java.util.stream.Collectors;

/**
 * O采购入库确认单(委外加工类型)->U8C委外入库单(存在多笔的情况)
 * <p>
 * 2025 年 3 月 11 日 15:57:47
 * 接入 O 正式环境拉取 O 单据我已经和妮姐电话+微信文字确认，推送到 U8C 的委外入库单（红蓝字）没有少字段的的情况，U8C 单据符合委外业务要求
 *
 * @Author：liuyang
 * @Package：com.hzya.frame.plugin.lets.plugin.outsourc
 * @Project：kangarooDataCenterV3
 * @name：ConsignmachiningIn
 * @Date：2024年8月26日 10:23:20
 * @Filename：ConsignmachiningIn
 */
public class ConsignmachiningCinfurmIn extends PluginBaseEntity {

    private static final ReentrantLock LOCK1 = new ReentrantLock(true);

    Logger logger = LoggerFactory.getLogger(ConsignmachiningCinfurmIn.class);

    @Autowired
    private QueryBdBusitypeUtil queryBdBusitypeUtil;

    @Autowired
    private IBdCorpDao iBdCorpDao;

    @Autowired
    private IBdInvmandocDao iBdInvmandocDao;

    @Autowired
    private IBdCalbodyDao iBdCalbodyDao;

    @Autowired
    private IBdStordocDao iBdStordocDao;

    @Autowired
    private RdclUtil rdclUtil;

    @Autowired
    private OfsStandardUtil ofsStandardUtil;

    @Autowired
    private IScOrderDao iScOrderDao;

    @Autowired
    private IScOrderBDao iScOrderBDao;

    @Autowired
    private IBdPurorgDao iBdPurorgDao;

    @Autowired
    private IBdCubasdocDao iBdCubasdocDao;

    @Autowired
    private IBdCumandocDao iBdCumandocDao;

    @Autowired
    private IBdInvbasdocDao iBdInvbasdocDao;

    @Autowired
    private OnlyImplementProxyOrderUtil onlyImplementProxyOrder;

    @Autowired
    private IBdBomDao iBdBomDao;

    @Autowired
    private SaveOrUpdateBusinessLogUtil saveOrUpdateBusinessLogUtil;

    @Autowired
    private IIntegrationTaskLivingDetailsDao iIntegrationTaskLivingDetailsDao;

    @Autowired
    private IntegrationTaskLivingDetailsUtil integrationTaskLivingDetailsUtil;

    @Autowired
    private OffsetTimeTime offsetTimeTime;

    private static final String PROD_FILED = "prod";

    @Autowired
    private QueryU8CEntityUtil queryU8CEntityUtil;

    @Autowired
    private IBdDeptdocDao iBdDeptdocDao;

    @Override
    public void initialize() {
        logger.info(getPluginLabel() + "執行初始化方法initialize()");
    }

    @Override
    public void destroy() {
        logger.info(getPluginLabel() + "執行銷毀方法destroy()");
    }

    @Override
    public String getPluginId() {
        return "outsourc.ConsignmachiningCinfurmIn";
    }

    @Override
    public String getPluginName() {
        return "O采购入库确认单(委外加工类型)生成U8C委外入库单(关联原单)";
    }

    @Override
    public String getPluginLabel() {
        return "O采购入库确认单(委外加工类型)生成U8C委外入库单(关联原单)";
    }

    @Override
    public String getPluginType() {
        return "3";
    }

    @Override
    public JsonResultEntity executeBusiness(JSONObject requestJson) throws Exception {
        logger.info("触发调用：{}" + getPluginName());
        try {
            Thread thread = new Thread(new Runnable() {
                @Override
                public void run() {
                    long startMillis = System.currentTimeMillis();
                    logger.info("executeBusiness方法调用开始：" + getPluginName() + "-插件");

                    try {
                        String param = String.valueOf(requestJson.get("param"));
                        if (ProfilesActiveConstant.TYPE_VBILLCODE.equals(requestJson.get("type"))) {
                            //按单号推送
                            if (param != null && !"".equals(param)) {
                                startImplementByCode(param);
                            }
                        } else if (ProfilesActiveConstant.TYPE_TIME_FRAME.equals(requestJson.get("type"))) {
                            //前台页面功能日期推送，或者接口调用也行
                            String craeteDateStr = offsetTimeTime.computingTime(param);
                            if (craeteDateStr != null && !"".equals(craeteDateStr)) {
                                splitDateAndPush(craeteDateStr);
                            }
                        } else if (ProfilesActiveConstant.TYPE_DETAIL_ERROR.equals(requestJson.get("type"))) {
                            String integration_task_living_details_id = String.valueOf(requestJson.get("integration_task_living_details_id"));
                            Assert.notNull(integration_task_living_details_id, "明细行主键不能为空！");
                            IntegrationTaskLivingDetailsEntity integrationTaskLivingDetailsEntity = saveOrUpdateBusinessLogUtil.queryIntegrationTaskLivingDetailsEntity(integration_task_living_details_id);
                            if (integrationTaskLivingDetailsEntity != null && integrationTaskLivingDetailsEntity.getRootAppBill() != null) {
                                startImplementByCode(integrationTaskLivingDetailsEntity.getRootAppBill());
                            }
                        } else {
                            if (ProfilesActiveConstant.LETS_PROFILES_ACTIVE.equals(PROD_FILED)) {
                                //默认被定时器执行，实时执行，计算时间偏移量
                                StartAndEndVo startAndEndVo = offsetTimeTime.getCurrentDateRange();
                                startImplementByTime(startAndEndVo.getStart_time(), startAndEndVo.getEnd_time());
                            }
                        }
                    } catch (Exception e) {
                        logger.error("executeBusiness-方法异常", e);
                    }
                    long endMillis = System.currentTimeMillis();
                    logger.info("executeBusiness-方法调用结束：" + getPluginName() + "-插件 执行耗时：{}", (endMillis - startMillis));
                }
            }, "执行插件：" + getPluginName());
            thread.start();

            if (requestJson != null && requestJson.get("type") != null && ProfilesActiveConstant.TYPE_DETAIL_ERROR.equals(requestJson.get("type"))) {
                try {
                    thread.join();
                } catch (Exception e) {
                    logger.error("thread.join();抛出异常", e);
                }
            }
        } catch (Exception e) {
            logger.error("executeBusiness方法抛出异常", e);
        }
        return BaseResult.getSuccessMessageEntity("插件执行成功");
    }

    /**
     * 根据时间范围查询
     *
     * @author liuyang
     */
    public void startImplementByTime(String startTime, String endTime) {
        long startMillis = System.currentTimeMillis();
        String threadNameStrStart = StrUtil.format("开始（委外采购）OFS采购入库单生成U8C委外加工入库 开始时间：{} 结束时间：{}", startTime, endTime);
        logger.info(threadNameStrStart);

        Thread thread = new Thread(new Runnable() {
            @Override
            public void run() {
                try {
                    List<OFSReceiptConfirmSearchResponse.Order> ofsWareConfirmaDataArrayList = new ArrayList<>();
                    QueryOfsSoSaleOutVo queryOfsSoSaleOutVo = new QueryOfsSoSaleOutVo();
                    queryOfsSoSaleOutVo.setReceivedAt_start(startTime);
                    queryOfsSoSaleOutVo.setReceivedAt_end(endTime);
                    queryOfsSoSaleOutVo.setClientCode("LETS");
                    queryOfsSoSaleOutVo.setPageNo(1L);
                    queryOfsSoSaleOutVo.setPageSize(50L);
                    queryOfsSoSaleOutVo.setInternalInstructionType("PURCHASE");
                    queryOfsSoSaleOutVo.setStatus(900L);
                    ofsStandardUtil.queryOfsWarehouseConfirOrder(queryOfsSoSaleOutVo, ofsWareConfirmaDataArrayList, 1L, "ofs.receipt.confirm.search");
                    logger.info("插件：{} O接口返回行数：{} 时间段：{} {}", getPluginName(), ofsWareConfirmaDataArrayList.size(), startTime, endTime);
                    printOfsDocCode(ofsWareConfirmaDataArrayList, startTime, endTime);

                    if (ofsWareConfirmaDataArrayList.size() > 0) {
                        String filter_company = queryU8CEntityUtil.queryBdDefDocByListPk();
                        ofsWareConfirmaDataArrayList.removeIf(new Predicate<OFSReceiptConfirmSearchResponse.Order>() {
                            @Override
                            public boolean test(OFSReceiptConfirmSearchResponse.Order headerDetailsDto) {
                                OFSReceiptConfirmSearchResponse.Header header = headerDetailsDto.getHeader();
                                if (header != null && header.getCompanyCode() != null) {
                                    if (filter_company.contains(header.getCompanyCode().trim())) {
                                        return true;
                                    }
                                }
                                return false;
                            }
                        });
                        getSet(ofsWareConfirmaDataArrayList);
                    } else {
                        logger.info("没有查询到任何数据！不需要同步");
                    }
                } catch (Exception e) {
                    logger.error("startImplement-方法抛出异常", e);
                }
            }
        }, threadNameStrStart);
        thread.start();
        try {
            thread.join();
        } catch (Exception e) {
            logger.error("thread.join()异常", e);
        }
        long endMillis = System.currentTimeMillis();
        String threadNameStrEnd = StrUtil.format("结束（委外采购）OFS采购入库单生成U8C委外加工入库 开始时间：{} 结束时间：{} 耗时：{}", startTime, endTime, (endMillis - startMillis));
        logger.info(threadNameStrEnd);
    }

    /**
     * 根据单号查询
     *
     * @author liuyang
     */
    public void startImplementByCode(String code) {
        long startMillis = System.currentTimeMillis();
        String threadNameStrStart = StrUtil.format("开始（委外采购）OFS采购入库单生成U8C委外加工入库 OFS采购入库单号：{}", code);
        logger.info(threadNameStrStart);
        Assert.notNull(code, "code不能为空");

        Thread thread = new Thread(new Runnable() {
            @Override
            public void run() {
                try {
                    List<OFSReceiptConfirmSearchResponse.Order> ofsWareConfirmaDataArrayList = new ArrayList<>();
                    QueryOfsSoSaleOutVo queryOfsSoSaleOutVo = new QueryOfsSoSaleOutVo();
                    queryOfsSoSaleOutVo.setCode(code);
                    queryOfsSoSaleOutVo.setClientCode("LETS");
                    queryOfsSoSaleOutVo.setPageNo(1L);
                    queryOfsSoSaleOutVo.setPageSize(50L);
                    queryOfsSoSaleOutVo.setInternalInstructionType("PURCHASE");
//                    queryOfsSoSaleOutVo.setStatus(900L);
                    ofsStandardUtil.queryOfsWarehouseConfirOrder(queryOfsSoSaleOutVo, ofsWareConfirmaDataArrayList, 1L, "ofs.receipt.confirm.search");
                    logger.info("插件：{} O接口返回行数：{} 单据号：{}", getPluginName(), ofsWareConfirmaDataArrayList.size(), code);
                    printOfsDocCode(ofsWareConfirmaDataArrayList, code);
                    logger.info("数据返回行数：{}", ofsWareConfirmaDataArrayList.size());

                    if (ofsWareConfirmaDataArrayList.size() > 0) {
                        String filter_company = queryU8CEntityUtil.queryBdDefDocByListPk();

                        ofsWareConfirmaDataArrayList.removeIf(new Predicate<OFSReceiptConfirmSearchResponse.Order>() {
                            @Override
                            public boolean test(OFSReceiptConfirmSearchResponse.Order headerDetailsDto) {
                                OFSReceiptConfirmSearchResponse.Header header = headerDetailsDto.getHeader();
                                if (header != null && header.getCompanyCode() != null) {
                                    if (filter_company.contains(header.getCompanyCode().trim())) {
                                        return true;
                                    }
                                }
                                return false;
                            }
                        });
                        getSet(ofsWareConfirmaDataArrayList);
                    } else {
                        logger.info("没有查询到任何数据！不需要同步");
                    }
                } catch (Exception e) {
                    logger.error("startImplement-方法抛出异常", e);
                }
            }
        }, threadNameStrStart);
        thread.start();
        try {
            thread.join();
        } catch (Exception e) {
            logger.error("thread.join()异常", e);
        }
        long endMillis = System.currentTimeMillis();
        String threadNameStrEnd = StrUtil.format("结束（委外采购）OFS采购入库单生成U8C委外加工入库 OFS采购入库单号：{}", code, (endMillis - startMillis));
        logger.info(threadNameStrEnd);
    }

    /**
     * 环境预配置
     *
     * @param ofsWareConfirmaDataArrayList 采购入库确认单
     * @author liuyang
     */
    private void getSet(List<OFSReceiptConfirmSearchResponse.Order> ofsWareConfirmaDataArrayList) throws Exception {
        LOCK1.lock();
        try {
            //过滤成功的数据
            List<OFSReceiptConfirmSearchResponse.Order> ofsWareConfirmaData = filterData(ofsWareConfirmaDataArrayList);
            //执行推送主逻辑
            implement(ofsWareConfirmaData);
        } catch (Exception e) {
            logger.error("getSet方法抛出异常", e);
        } finally {
            LOCK1.unlock();
        }
    }

    /**
     * 过滤掉成功的数据
     *
     * @author liuyang
     */
    private List<OFSReceiptConfirmSearchResponse.Order> filterData(List<OFSReceiptConfirmSearchResponse.Order> ofsWareConfirmaDataList) {
        //过滤完毕后的数据集合
        List<OFSReceiptConfirmSearchResponse.Order> headerDetailsDtoList1 = new ArrayList<>();
        //查询得到已经成功的日志，成功或者已处理
        List<IntegrationTaskLivingDetailsEntity> successIntegrationTaskLivingDetails = new ArrayList<>();
        try {
            if (ofsWareConfirmaDataList != null && ofsWareConfirmaDataList.size() > 0) {
                //每100个开始拆分
                List<List<OFSReceiptConfirmSearchResponse.Order>> lists = SplitListByCountUtil.splitListByCount(ofsWareConfirmaDataList, 500);
                for (int i = 0; i < lists.size(); i++) {
                    List<OFSReceiptConfirmSearchResponse.Order> ofsWareConfirmaData = lists.get(i);
                    String primaryKey = fieldConcaten(ofsWareConfirmaData);

                    List<IntegrationTaskLivingDetailsEntity> integrationTaskLivingDetailsEntities = integrationTaskLivingDetailsUtil.queryIntegrationTaskLivingDetails(primaryKey, getPluginId());
                    successIntegrationTaskLivingDetails.addAll(integrationTaskLivingDetailsEntities);
                }

                for (int i = 0; i < ofsWareConfirmaDataList.size(); i++) {
                    OFSReceiptConfirmSearchResponse.Order ofsWareConfirmaData = ofsWareConfirmaDataList.get(i);
                    OFSReceiptConfirmSearchResponse.Header header = ofsWareConfirmaData.getHeader();
                    List<OFSReceiptConfirmSearchResponse.Detail> details = ofsWareConfirmaData.getDetails();

                    boolean isExi = false;
                    for (int j = 0; j < successIntegrationTaskLivingDetails.size(); j++) {
                        IntegrationTaskLivingDetailsEntity integrationTaskLivingDetailsEntity = successIntegrationTaskLivingDetails.get(j);
                        if (integrationTaskLivingDetailsEntity.getRootAppPk().equals(header.getId())) {
                            isExi = true;
                        }
                    }
                    if (!isExi) {
                        headerDetailsDtoList1.add(ofsWareConfirmaData);
                    }
                }
            }
        } catch (Exception e) {
            logger.error("过滤成功的数据抛出异常", e);
        }
        return headerDetailsDtoList1;
    }

    /**
     * 字段拼接
     */
    private String fieldConcaten(List<OFSReceiptConfirmSearchResponse.Order> headerDetailsDtoList1) {
        if (headerDetailsDtoList1 != null && headerDetailsDtoList1.size() > 0) {
            StringBuilder result = new StringBuilder();
            for (OFSReceiptConfirmSearchResponse.Order obj : headerDetailsDtoList1) {
                OFSReceiptConfirmSearchResponse.Header header = obj.getHeader();
                result.append("'").append(header.getId()).append("'").append(",");
            }
            return result.substring(0, result.length() - 1);
        }
        return null;
    }

    /**
     * 执行主逻辑
     *
     * @author liuyang
     */
    private void implement(List<OFSReceiptConfirmSearchResponse.Order> ofsWareConfirmaData) throws Exception {
        if (ofsWareConfirmaData != null && ofsWareConfirmaData.size() > 0) {
            try {
                //查询基本档案
                List<PurchaseReceiptConfirmationSonDto> purchaseReceiptConfirmationSonDtos = queryBasicArchives(ofsWareConfirmaData);
                //查询采购收发类别
                BdRdclEntity bdRdclEntity = rdclUtil.queryRdClObject("105");
                //代理委外入库业务流程
                BdBusitypeEntity bdBusitypeEntity = u8cOperationFlow();

                if (purchaseReceiptConfirmationSonDtos != null && purchaseReceiptConfirmationSonDtos.size() > 0) {
                    for (int i = 0; i < purchaseReceiptConfirmationSonDtos.size(); i++) {
                        PurchaseReceiptConfirmationSonDto purchaseReceiptConfirmationSonDto = purchaseReceiptConfirmationSonDtos.get(i);

                        String generateBusinessDate = null;
                        try {
                            generateBusinessDate = createGenerateBusinessDate(purchaseReceiptConfirmationSonDto); //生成业务日期

                            OfsPoOrderData ofsPoOrderData = purchaseReceiptConfirmationSonDto.getOfsPoOrderData();//OFS采购订单
                            OfsPoOrderHeader ofsPoOrderDataHeader = ofsPoOrderData.getHeader();
                            List<OfsPoOrderDetails> ofsPoOrderDataDetails = ofsPoOrderData.getDetails();

                            StockinOrderSearchResponse.StockinOrder stockinOrder = purchaseReceiptConfirmationSonDto.getStockinOrder();//OFS采购入库单

                            List<OFSReceiptConfirmSearchResponse.Detail> details = purchaseReceiptConfirmationSonDto.getDetails();//OFS采购入库确认单明细行
                            BdCorpEntity bdCorpEntity = purchaseReceiptConfirmationSonDto.getBdCorpEntity();//公司
                            BdCalbodyEntity bdCalbodyEntity = purchaseReceiptConfirmationSonDto.getBdCalbodyEntity();//库存组织
                            BdPurorgEntity bdPurorgEntity = purchaseReceiptConfirmationSonDto.getBdPurorgEntity();//采购组织
                            BdCubasdocEntity bdCubasdocEntity = purchaseReceiptConfirmationSonDto.getBdCubasdocEntity();//客商基本档案
                            BdCumandocEntity bdCumandocEntity = purchaseReceiptConfirmationSonDto.getBdCumandocEntity();//客商管理档案
                            BdStordocEntity bdStordocEntity = purchaseReceiptConfirmationSonDto.getBdStordocEntity();//表体仓库
                            BdDeptdocEntity bdDeptdocEntity = purchaseReceiptConfirmationSonDto.getBdDeptdocEntity();//采购部门
                            ScOrderEntity scOrderEntity = purchaseReceiptConfirmationSonDto.getScOrderEntity();//U8C 委外订单
                            List<ScOrderBEntity> scOrderBEntityList = purchaseReceiptConfirmationSonDto.getScOrderBEntityList();//U8C 委外订单明细行

                            //委外订单表头
                            ConsignmachiningInHeadDto poOrderParentDto = new ConsignmachiningInHeadDto();
                            poOrderParentDto.setPk_corp(bdCorpEntity.getPkCorp());
                            poOrderParentDto.setCoperatorid(OverallConstant.getOverAllValue("u8cApiZdrPK"));
                            poOrderParentDto.setCwarehouseid(bdStordocEntity.getPkStordoc());
                            poOrderParentDto.setPk_calbody(bdCalbodyEntity.getPkCalbody());
                            poOrderParentDto.setCbiztype(bdBusitypeEntity.getPkBusitype());
                            poOrderParentDto.setCdispatcherid(bdRdclEntity.getPkRdcl());//收发类别
                            poOrderParentDto.setDbilldate(generateBusinessDate);//入库确认日期
                            poOrderParentDto.setFreplenishflag(false);//是否退货
                            poOrderParentDto.setCdptid(bdDeptdocEntity.getPkDeptdoc());//采购部门
                            poOrderParentDto.setCproviderid(bdCumandocEntity.getPkCumandoc());//加工单位
                            poOrderParentDto.setVuserdef17(ProfilesActiveConstant.sourceSystem1);//来源系统
                            poOrderParentDto.setVuserdef19(purchaseReceiptConfirmationSonDto.getCode());//原单单号=OFS 采购入库单确认单-单号
                            poOrderParentDto.setVuserdef20(purchaseReceiptConfirmationSonDto.getId());//原单主键=OFS 采购入库单确认单-主键

                            //委外入库表头明细行
                            List<ConsignmachiningInBodyDto> consignmachiningInBodyDtoArrayList = new ArrayList<>();
                            for (int j = 0; j < details.size(); j++) {
                                OFSReceiptConfirmSearchResponse.Detail detail = details.get(j);

                                //如果明细行的实发数量为0则过滤掉
                                if ("0".equals(new BigDecimal(detail.getReceivedQty()).stripTrailingZeros().toPlainString())) {
                                    continue;
                                }

                                //匹配U8C采购订单明细行
                                ScOrderBEntity u8cPoOrderDetail = findOfsPoOrderDetailAndU8cPoOrderDetail(detail, stockinOrder, ofsPoOrderData, scOrderBEntityList);
                                //通过O采购订单入库单明细行，联查OFS采购入库单明细行
//                                StockinOrderSearchResponse.StockinOrder.StockinB ofsPoOrderDetailRowTraget = findOfsPoOrderInDetailRowTraget(detail, stockinOrder);
                                //通过O采购入库单明细行
//                                OfsPoOrderDetails ofsPoOrderDetailRowTraget1 = findOfsPoOrderDetailRowTraget(ofsPoOrderDetailRowTraget, ofsPoOrderData);
                                //根据O入库确认单明细行，联查O采购订单明细行
                                OfsPoOrderDetails ofsPoOrderDetailAndU8cPoOrderDetail = findOfsPoOrderDetailAndU8cPoOrderDetail(detail, ofsPoOrderData);

                                //查询存货管理档案
                                BdInvmandocEntity bdInvmandocEntity = queryInventoryMan(detail, bdCorpEntity.getPkCorp());
                                //存货基础档案
                                BdInvbasdocEntity bdInvbasdocEntity = queryStockBasicArchives(bdInvmandocEntity.getPkInvmandoc(), bdCorpEntity.getPkCorp());
                                //存货对应的BOM
//                                List<BdBomEntity> bomVersion = getBomVersion(bdCalbodyEntity.getPkCalbody(), bdInvbasdocEntity.getPkInvbasdoc(), bdInvbasdocEntity.getInvcode());

                                //计算含税单价
                                BigDecimal noriginalcurprice = null;
                                //实收数量
                                String receivedQty = detail.getReceivedQty();
                                //实收累计总金额
                                String fulfillAmount = ofsPoOrderDetailAndU8cPoOrderDetail.getDiscountPrice();
                                try {
                                    noriginalcurprice = new BigDecimal(fulfillAmount).divide(new BigDecimal(receivedQty), 20, BigDecimal.ROUND_HALF_UP).setScale(4, BigDecimal.ROUND_HALF_UP);
                                } catch (Exception e) {
                                    logger.error("计算采退出库单原币含税单价失败", e);
                                    Assert.state(false, "计算采退出库单原币含税单价失败 原因：{}", e.getMessage());
                                }

                                ConsignmachiningInBodyDto consignmachiningInBodyDto = new ConsignmachiningInBodyDto();
                                consignmachiningInBodyDto.setCinventoryid(bdInvmandocEntity.getPkInvmandoc());//存货主键
//                                if ("40756".equals(new BigDecimal(receivedQty).stripTrailingZeros().toPlainString())) {
//                                    System.out.println("111");
//                                }
                                consignmachiningInBodyDto.setNinnum(new BigDecimal(receivedQty).stripTrailingZeros().toPlainString());//实收数量
                                consignmachiningInBodyDto.setNprice(noriginalcurprice.stripTrailingZeros().toPlainString());//单价（含税单价）
                                consignmachiningInBodyDto.setCsourcetype("61");//来源单据类型
                                consignmachiningInBodyDto.setVsourcerowno(u8cPoOrderDetail.getCrowno());//来源单据行号
//                                consignmachiningInBodyDto.setVsourcebillcode(scOrderEntity.getVordercode());//来源单据号
//                            consignmachiningInBodyDto.setWriteofftype("按备料发料核销");//核销方式，和妮姐已经确认，不用传递
                                //2025年2月13日 17:36:28 和妮姐电话确认，核销方式永远为按BOM核销，问了2遍
                                consignmachiningInBodyDto.setWriteofftype("按BOM核销");//核销方式，和妮姐已经确认，不用传递
                                consignmachiningInBodyDto.setDbizdate(generateBusinessDate);//	入库日期
//                                consignmachiningInBodyDto.setVdef16(bomVersion.get(0).getVersion());//BOM version
                                //2025年2月13日 17:35:45 和妮姐电话确认，bom版本永远为1.0，问了3遍
                                consignmachiningInBodyDto.setBomversion("1.0");//BOM version
                                consignmachiningInBodyDto.setCsourcebillbid(u8cPoOrderDetail.getCorderBid());//来源明细主键
                                consignmachiningInBodyDto.setCsourcebillhid(scOrderEntity.getCorderid());//来源表头主键

                                consignmachiningInBodyDto.setCfirstbillbid(u8cPoOrderDetail.getCorderBid());//源头单据表体id
                                consignmachiningInBodyDto.setCfirstbillhid(scOrderEntity.getCorderid());//源头单据表头id
                                consignmachiningInBodyDto.setCfirsttype("61");//源头单据类型
                                consignmachiningInBodyDtoArrayList.add(consignmachiningInBodyDto);
                            }

                            //加个小验证,如果Cfirstbillbid、Cfirstbillhid、Cfirsttype其中之一没有传，则抛出异常
                            for (int j = 0; j < consignmachiningInBodyDtoArrayList.size(); j++) {
                                ConsignmachiningInBodyDto consignmachiningInBodyDto = consignmachiningInBodyDtoArrayList.get(j);
                                if (consignmachiningInBodyDto.getCfirstbillbid() == null || "".equals(consignmachiningInBodyDto.getCfirstbillbid())) {
                                    Assert.state(false, "cfirstbillbid不能为空！");
                                }
                                if (consignmachiningInBodyDto.getCfirstbillhid() == null || "".equals(consignmachiningInBodyDto.getCfirstbillhid())) {
                                    Assert.state(false, "cfirstbillhid不能为空！");
                                }
                                if (consignmachiningInBodyDto.getCfirsttype() == null || "".equals(consignmachiningInBodyDto.getCfirsttype())) {
                                    Assert.state(false, "cfirsttype不能为空！");
                                }
                            }

                            List<ConsignmachiningInHeadBodyDto> consignmachiningInHeadBodyDtoArrayList = new ArrayList<>();
                            ConsignmachiningInHeadBodyDto consignmachiningInHeadBodyDto = new ConsignmachiningInHeadBodyDto();
                            consignmachiningInHeadBodyDto.setParentvo(poOrderParentDto);
                            consignmachiningInHeadBodyDto.setChildrenvo(consignmachiningInBodyDtoArrayList);
                            consignmachiningInHeadBodyDtoArrayList.add(consignmachiningInHeadBodyDto);

                            Map<String, List<ConsignmachiningInHeadBodyDto>> stringStringMap = new HashMap<>();
                            stringStringMap.put("GeneralBillVO", consignmachiningInHeadBodyDtoArrayList);
                            //2024年8月26日 14:41:46 委外入库核销方式传不了！已经提交总部，2024年8月30日 14:05:25 已经解决，和妮姐确认不用填写
                            //2024年8月26日 14:42:19 无法超委外订单数量入库！已经提交给总部
                            //2024年9月3日 18:28:24 退货不会出现超订单出库的情况
                            //2025-05-15 15:51:04 记录日志信息
                            String interfaceInputParameters = JSON.toJSONString(stringStringMap);
                            logger.info("O采购入库确认单(委外加工类型)生成U8C委外入库单(关联原单) 接口入参：{}", interfaceInputParameters);
                            IcGeneralHResultDto icGeneralHResultDto = sendU8cConsignmachiningIn(interfaceInputParameters);

                            String cgeneralhid = null;
                            String vbillcode = null;
                            IcGeneralHResultHeadDto parentvo = icGeneralHResultDto.getParentvo();
                            List<IcGeneralHResultBodyDto> childrenvo = icGeneralHResultDto.getChildrenvo();
                            if (parentvo != null) {
                                cgeneralhid = parentvo.getCgeneralhid();
                                vbillcode = parentvo.getVbillcode();
                            }
                            logger.info("推送U8C委外入库成功！委外入库单主键：{} 委外入库单编码：{} 公司名称：{}", cgeneralhid, vbillcode, bdCorpEntity.getUnitname());

                            // 成功记录日志
                            IntegrationTaskLivingDetailsEntity integrationTaskLivingDetailsEntity = new IntegrationTaskLivingDetailsEntity();
                            integrationTaskLivingDetailsEntity.setNewState(ProfilesActiveConstant.LOG_STATUS_Y);
                            integrationTaskLivingDetailsEntity.setRootAppNewData(JSON.toJSONString(purchaseReceiptConfirmationSonDto));//原始数据json
                            integrationTaskLivingDetailsEntity.setNewTransmitInfo(JSON.toJSONString(icGeneralHResultDto));//返回结果
                            integrationTaskLivingDetailsEntity.setNewPushDate(new Date());
                            integrationTaskLivingDetailsEntity.setBusinessDate(generateBusinessDate);
                            integrationTaskLivingDetailsEntity.setRootAppPk(purchaseReceiptConfirmationSonDto.getId());
                            integrationTaskLivingDetailsEntity.setRootAppBill(purchaseReceiptConfirmationSonDto.getCode());
                            integrationTaskLivingDetailsEntity.setPluginId(getPluginId());
                            integrationTaskLivingDetailsEntity.setNewSystemNumber(vbillcode);//下游单据编码
                            integrationTaskLivingDetailsEntity.setNewSystemPrimary(cgeneralhid);//下游单据主键
                            saveOrUpdateBusinessLogUtil.saveOrUpdate(integrationTaskLivingDetailsEntity);
                        } catch (Exception e) {
                            logger.error("推送U8C委外订单失败", e);

                            String message = e.getMessage();
                            if (message == null) {
                                message = "未知错误";
                            }
                            // 失败记录日志
                            IntegrationTaskLivingDetailsEntity integrationTaskLivingDetailsEntity = new IntegrationTaskLivingDetailsEntity();
                            integrationTaskLivingDetailsEntity.setNewState(ProfilesActiveConstant.LOG_STATUS_N);
                            integrationTaskLivingDetailsEntity.setRootAppNewData(JSON.toJSONString(purchaseReceiptConfirmationSonDto));//原始数据json
                            integrationTaskLivingDetailsEntity.setNewTransmitInfo(JSON.toJSONString(message));//返回结果
                            integrationTaskLivingDetailsEntity.setNewPushDate(new Date());
                            integrationTaskLivingDetailsEntity.setBusinessDate(generateBusinessDate);
                            integrationTaskLivingDetailsEntity.setRootAppPk(purchaseReceiptConfirmationSonDto.getId());
                            integrationTaskLivingDetailsEntity.setRootAppBill(purchaseReceiptConfirmationSonDto.getCode());
                            integrationTaskLivingDetailsEntity.setPluginId(getPluginId());
                            saveOrUpdateBusinessLogUtil.saveOrUpdate(integrationTaskLivingDetailsEntity);
                        }
                    }
                } else {
                    logger.info("purchaseReceiptConfirmationSonDtos.size为0！");
                }
            } catch (Exception e) {
                logger.error("implement方法抛出异常", e);
            }
        }
    }

    /**
     * 档案转换(OFS->U8C)
     * 2024年8月23日 12:02:47 和妮姐沟通，单据流程U8C成品委外订单 -> 备料计划 -> U8C委外发料(核销)
     *
     * @author liuyang
     */
    private List<PurchaseReceiptConfirmationSonDto> queryBasicArchives(List<OFSReceiptConfirmSearchResponse.Order> returnGoodHeaderDetailsDataDtoList) throws Exception {
        List<PurchaseReceiptConfirmationSonDto> consignmachiningInSonDtoArrayList = new ArrayList<>();

        if (returnGoodHeaderDetailsDataDtoList != null && returnGoodHeaderDetailsDataDtoList.size() > 0) {
            try {
                //批量查询O采购入库单
                queryBatchOfsPoOrderIn(returnGoodHeaderDetailsDataDtoList);
                //批量查询O采购订单
                queryBatchOfsPoOrder(returnGoodHeaderDetailsDataDtoList);
            } catch (Exception e) {
                logger.error("方法queryBasicArchives抛出异常", e);
            }

            for (int i = 0; i < returnGoodHeaderDetailsDataDtoList.size(); i++) {
                OFSReceiptConfirmSearchResponse.Order ofsWareConfirmaData = returnGoodHeaderDetailsDataDtoList.get(i);
                OFSReceiptConfirmSearchResponse.Header header = ofsWareConfirmaData.getHeader();
                List<OFSReceiptConfirmSearchResponse.Detail> details = ofsWareConfirmaData.getDetails();

                String generateBusinessDate = null;
                try {
                    //生成业务日期
                    generateBusinessDate = createGenerateBusinessDateV2(header);

                    //2025 年 3 月 10 日 10:58:59 查询 ofs 采购入库单
//                    String outBizCode = header.getCode();
//                    Assert.notNull(outBizCode, "采购入库确定单对应的采购入库单号不能为空 outBizCode字段:{}", outBizCode);
//                    Assert.state(!"".equals(outBizCode), "采购入库确定单对应的采购入库单号不能为空 outBizCode字段;{}", outBizCode);
//                    outBizCode = outBizCode.replace("-confirm", "");

//                    List<StockinOrderSearchResponse.StockinOrder> returnGoodHeaderDetailsDataDtoArrayList = new ArrayList<>();
//                    QueryOfsSoSaleOutVo queryOfsSoSaleOutVo = new QueryOfsSoSaleOutVo();
//                    queryOfsSoSaleOutVo.setClientCode("LETS");
//                    queryOfsSoSaleOutVo.setPageNo(1L);
//                    queryOfsSoSaleOutVo.setPageSize(50L);
//                    queryOfsSoSaleOutVo.setCode(outBizCode);
//                    queryOfsSoSaleOutVo.setInternalInstructionType("PURCHASE");
//                    ofsStandardUtil.queryOfsReturnGoods(queryOfsSoSaleOutVo, returnGoodHeaderDetailsDataDtoArrayList, 1L, "ofs.receipt.search");
//                    if (returnGoodHeaderDetailsDataDtoArrayList == null || returnGoodHeaderDetailsDataDtoArrayList.size() == 0) {
//                        Assert.state(false, "根据采购入库单号：{}无法查询到数据", outBizCode);
//                    }
//                    StockinOrderSearchResponse.StockinOrder stockinOrder = returnGoodHeaderDetailsDataDtoArrayList.get(0);
//                    StockinOrderSearchResponse.StockinOrder.StockinH header1 = stockinOrder.getHeader();
//                    List<StockinOrderSearchResponse.StockinOrder.StockinB> details1 = stockinOrder.getDetails();

                    Assert.notNull(header.getOfsStockinOrder(), "无法查询到对应的O采购入库单!");
                    StockinOrderSearchResponse.StockinOrder stockinOrder = header.getOfsStockinOrder();
                    StockinOrderSearchResponse.StockinOrder.StockinH header1 = stockinOrder.getHeader();
                    List<StockinOrderSearchResponse.StockinOrder.StockinB> details1 = stockinOrder.getDetails();

                    //2024年8月25日 09:54:31 查询OFS采购订单
//                    OfsPoOrderData ofsPoOrderData = ofsStandardUtil.queryOfsPoOrder(header1.getRefOrderCode());
//                    Assert.notNull(ofsPoOrderData, "无法查询到委外订单(OFS采购入库单->OSF采购订单->OFS采购订单主键->U8C委外订单自定义项20)");
//                    OfsPoOrderHeader ofsPoOrderDataHeader = ofsPoOrderData.getHeader();
//                    List<OfsPoOrderDetails> ofsPoOrderDataDetails = ofsPoOrderData.getDetails();
                    Assert.notNull(header.getOfsOrderData(), "没有查询到对应的O采购订单！");
                    OfsPoOrderData ofsPoOrderData = header.getOfsOrderData();
                    OfsPoOrderHeader ofsPoOrderDataHeader = ofsPoOrderData.getHeader();
                    List<OfsPoOrderDetails> ofsPoOrderDataDetails = ofsPoOrderData.getDetails();

                    //2025 年 3 月 10 日 11:36:18 前置的流程：O 采购入库单 生成 U8C 委外订单，我需要在 U8C 委外订单表头保存 O 采购入库单单号（code）
                    //当执行流程：O 采购入库确认单 生成 U8C 委外入库单时，查询轨迹：O 采购入库确认单 -> O 采购入库单 从而得到 O 采购入库单单号，把这个单号导入到 U8C 委外订单表头查询，目前是 vdef19
                    //从而得到 U8C 委外订单

                    //如果该O采购订单对应的类型为：委外加工，则继续代码逻辑，否则就跳过代码逻辑
                    boolean checkResult = onlyImplementProxyOrder.onlyImplementProxyOrder(ofsPoOrderData, "WWJG");
                    if (!checkResult) {
                        continue;
                    }

                    //查询 U8C 委外订单
                    ScOrderEntity scOrderEntity = queryScOrder(header1.getCode());
                    Assert.notNull(scOrderEntity, "查询不到 U8C 委外订单，O 采购入库单号：{} 委外订单可能传递失败！", header1.getCode());

                    //查询 U8C 委外订单明细行
                    List<ScOrderBEntity> scOrderBEntityList = queryScOrderDetail(scOrderEntity.getCorderid());
                    if (scOrderBEntityList == null || scOrderBEntityList.size() == 0) {
                        Assert.state(false, "根据委外订单主键：{} 无法查询到 U8C 委外订单明细行", scOrderEntity.getCorderid());
                    }

                    //委外订单表头公司
                    String companyCode = header.getCompanyCode();
                    Assert.notNull(companyCode, "O表头公司不能为空(物理货主)");
                    String replaceCompanyCode = queryU8CEntityUtil.queryBdDefDocByPkDefDocList(companyCode.trim());

                    BdCorpEntity bdCorpEntity = new BdCorpEntity();
                    bdCorpEntity.setDr(0);
                    bdCorpEntity.setDataSourceCode("lets_u8c");
                    bdCorpEntity.setUnitcode(replaceCompanyCode);
                    List<BdCorpEntity> bdCorpEntityList = iBdCorpDao.query(bdCorpEntity);
                    if (bdCorpEntityList.size() == 0) {
                        Assert.state(false, "根据O货主编码{}，无法匹配到U8C委外订单公司", replaceCompanyCode);
                    } else if (bdCorpEntityList.size() >= 2) {
                        Assert.state(false, "根据O货主编码{}，匹配到U8C委外订单公司{}个", replaceCompanyCode, bdCorpEntityList.size());
                    }

                    //采购组织
                    BdPurorgEntity bdPurorgEntity = new BdPurorgEntity();
                    bdPurorgEntity.setDr("0");
                    bdPurorgEntity.setDataSourceCode("lets_u8c");
                    bdPurorgEntity.setOwnercorp(bdCorpEntityList.get(0).getPkCorp());
                    List<BdPurorgEntity> bdPurorgEntityList = iBdPurorgDao.query(bdPurorgEntity);
                    if (bdPurorgEntityList.size() == 0) {
                        Assert.state(false, "根据委外订单公司主键：{} 无法匹配到采购组织", bdCorpEntityList.get(0).getPkCorp());
                    } else if (bdPurorgEntityList.size() >= 2) {
                        Assert.state(false, "根据委外订单公司主键：{} 匹配到多个采购组织", bdCorpEntityList.get(0).getPkCorp());
                    }

                    //客商基本档案(供应商类型)
                    //2024年8月20日 14:47:55 丽知商城、OFS 供应商客商档案、传递到U8C的客商，
                    //其中自定义项1作为原系统编码，因此统一传到自定义项一，已经和大家确认好了，没有关系的，放心大胆传吧，宝贝
                    String shipFromCode = header1.getShipFromCode();
                    Assert.notNull(shipFromCode, "O供应商编码不能为空，没有办法完成业务逻辑，请配置供应商编码(采购入库)");
                    Assert.state(!"".equals(shipFromCode.trim()), "O供应商编码不能为空，没有办法完成业务逻辑，请配置供应商编码！");
//                    BdCubasdocEntity bdCubasdocEntity = new BdCubasdocEntity();
//                    bdCubasdocEntity.setDataSourceCode("lets_u8c");
//                    bdCubasdocEntity.setDr(0L);
//                    bdCubasdocEntity.setDef1(shipFromCode);
//                    List<BdCubasdocEntity> bdCubasdocEntityList = iBdCubasdocDao.query(bdCubasdocEntity);
//                    if (bdCubasdocEntityList == null || bdCubasdocEntityList.size() == 0) {
//                        Assert.state(false, "根据OFS供应商业务编码{}(客商自定义项1记录O客商业务编码)，无法查询到U8C客商档案信息(供应商)", shipFromCode);
//                    } else if (bdCubasdocEntityList.size() >= 2) {
//                        Assert.state(false, "根据OFS供应商业务编码{}(客商自定义项1记录O客商业务编码)，查询到多个U8C客商档案信息(供应商)", shipFromCode);
//                    }
                    BdCubasdocEntity fuzzyQueryCustomers = queryU8CEntityUtil.getFuzzyQueryCustomers(shipFromCode.trim());
                    Assert.notNull(fuzzyQueryCustomers, "根据O供应商编码：{}，无法查询到U8C客商基本档案 def1", shipFromCode.trim());

                    //客商管理档案
                    BdCumandocEntity bdCumandocEntity1 = new BdCumandocEntity();
                    bdCumandocEntity1.setDataSourceCode("lets_u8c");
                    bdCumandocEntity1.setDr(0L);
                    bdCumandocEntity1.setPkCorp(bdCorpEntityList.get(0).getPkCorp());
                    bdCumandocEntity1.setCustflags(ProfilesActiveConstant.SUPPLIER);
                    bdCumandocEntity1.setPkCubasdoc(fuzzyQueryCustomers.getPkCubasdoc());
                    List<BdCumandocEntity> bdCumandocEntityList = iBdCumandocDao.query(bdCumandocEntity1);
                    if (bdCumandocEntityList == null || bdCumandocEntityList.size() == 0) {
                        Assert.state(false, "客商：{} 没有分配给：{}，或者客商属性为客户，不满足客商或者供应商的要求！", fuzzyQueryCustomers.getCustshortname(), bdCorpEntityList.get(0).getUnitname());
                    } else if (bdCumandocEntityList.size() >= 2) {
                        Assert.state(false, "客商：{} 公司：{} 存在多个对应关系！", fuzzyQueryCustomers.getCustshortname(), bdCorpEntityList.get(0).getUnitname());
                    }

                    //2024年8月25日 10:09:20 委外入库公司，取委外订单上的公司，已经和妮姐确认，没有必要通过O的公司转换，因为不存在跨组织的问题
//                    Assert.notNull(scOrderEntity.getPkCorp(), "O表头公司不能为空");
//                    BdCorpEntity bdCorpEntity = new BdCorpEntity();
//                    bdCorpEntity.setDr(0);
//                    bdCorpEntity.setDataSourceCode("lets_u8c");
//                    bdCorpEntity.setPkCorp(scOrderEntity.getPkCorp());
//                    List<BdCorpEntity> bdCorpEntityList = iBdCorpDao.query(bdCorpEntity);
//                    if (bdCorpEntityList.size() == 0) {
//                        Assert.state(false, "根据委外订单公司主键:{}，无法匹配到委外入库单", scOrderEntity.getPkCorp());
//                    } else if (bdCorpEntityList.size() >= 2) {
//                        Assert.state(false, "根据委外订单公司主键:{}，匹配到{}个委外入库单", scOrderEntity.getPkCorp(), bdCorpEntityList.size());
//                    }

                    //库存组织
                    BdCalbodyEntity bdCalbodyEntity = new BdCalbodyEntity();
                    bdCalbodyEntity.setDr(0);
                    bdCalbodyEntity.setDataSourceCode("lets_u8c");
                    bdCalbodyEntity.setPkCorp(bdCorpEntityList.get(0).getPkCorp());
                    List<BdCalbodyEntity> bdCalbodyEntities = iBdCalbodyDao.query(bdCalbodyEntity);
                    if (bdCalbodyEntities.size() == 0) {
                        Assert.state(false, "根据U8C采购公司{}，无法匹配到U8C发货库存组织", bdCorpEntityList.get(0).getPkCorp());
                    } else if (bdCalbodyEntities.size() >= 2) {
                        Assert.state(false, "根据U8C采购公司{}，匹配到U8C发货库存组织{}个", bdCorpEntityList.get(0).getPkCorp(), bdCalbodyEntities.size());
                    }

                    //表体仓库
                    String facilityCode = header.getFacilityCode();
                    Assert.notNull(facilityCode, "O售后入库单仓库facilityCode字段不能为空");
                    BdStordocEntity bdStordocEntity = new BdStordocEntity();
                    bdStordocEntity.setDr(0L);
                    bdStordocEntity.setDataSourceCode("lets_u8c");
                    bdStordocEntity.setPkCalbody(bdCalbodyEntities.get(0).getPkCalbody());
                    bdStordocEntity.setStorcode(facilityCode);
                    List<BdStordocEntity> bdStordocEntityList = iBdStordocDao.query(bdStordocEntity);
                    if (bdStordocEntityList.size() == 0) {
                        Assert.state(false, "根据O仓库编码+U8C收货库存组织主键，无法匹配到U8C仓库", facilityCode, bdCalbodyEntities.get(0).getPkCalbody());
                    } else if (bdStordocEntityList.size() >= 2) {
                        Assert.state(false, "根据O仓库编码+U8C收货库存组织主键，匹配到U8C仓库多个", facilityCode, bdCalbodyEntities.get(0).getPkCalbody());
                    }

                    //查询采购部门
                    String deptName = "业务部门";
                    BdDeptdocEntity bdDeptdocEntity = new BdDeptdocEntity();
                    bdDeptdocEntity.setDataSourceCode("lets_u8c");
                    bdDeptdocEntity.setDr(0);
                    bdDeptdocEntity.setDeptname(deptName);
                    bdDeptdocEntity.setPkCorp(bdCorpEntityList.get(0).getPkCorp());
                    List<BdDeptdocEntity> bdDeptdocEntityList = iBdDeptdocDao.query(bdDeptdocEntity);
                    if (bdDeptdocEntityList == null || bdDeptdocEntityList.size() == 0) {
                        Assert.state(false, "没有匹配到采购部门，或者没有分配，公司名称:{} 部门名称：{}", bdCorpEntityList.get(0).getUnitname(), deptName);
                    }

                    PurchaseReceiptConfirmationSonDto consignmachiningInSonDto = new PurchaseReceiptConfirmationSonDto();
                    consignmachiningInSonDto.setOfsPoOrderData(ofsPoOrderData);
                    consignmachiningInSonDto.setStockinOrder(stockinOrder);
//                    consignmachiningInSonDto.setScOrderEntity(scOrderEntity);
                    consignmachiningInSonDto.setBdCorpEntity(bdCorpEntityList.get(0));
                    consignmachiningInSonDto.setBdCalbodyEntity(bdCalbodyEntities.get(0));
//                    consignmachiningInSonDto.setBdStordocEntity(bdStordocEntityList.get(0));
                    consignmachiningInSonDto.setBdPurorgEntity(bdPurorgEntityList.get(0));
                    consignmachiningInSonDto.setBdCubasdocEntity(fuzzyQueryCustomers);
                    consignmachiningInSonDto.setBdCumandocEntity(bdCumandocEntityList.get(0));
//                    consignmachiningInSonDto.setHeader(header);
//                    consignmachiningInSonDto.setScOrderBEntityList(scOrderBEntityList);
                    consignmachiningInSonDto.setDetails(details);
                    consignmachiningInSonDto.setBdStordocEntity(bdStordocEntityList.get(0));
                    consignmachiningInSonDto.setBdDeptdocEntity(bdDeptdocEntityList.get(0));
                    consignmachiningInSonDto.setScOrderEntity(scOrderEntity);
                    consignmachiningInSonDto.setScOrderBEntityList(scOrderBEntityList);

                    BeanUtil.copyPropertiesV2(header, consignmachiningInSonDto);
                    consignmachiningInSonDtoArrayList.add(consignmachiningInSonDto);
                    //成功，交给下一个业务流程处理环节
                } catch (Exception e) {
                    logger.error("采购入库确认单OFS档案转换错误", e);

                    String message = e.getMessage();
                    if (message == null) {
                        message = "未知错误";
                    }

                    //失败
                    IntegrationTaskLivingDetailsEntity integrationTaskLivingDetailsEntity = new IntegrationTaskLivingDetailsEntity();
                    integrationTaskLivingDetailsEntity.setNewState(ProfilesActiveConstant.LOG_STATUS_N);
                    integrationTaskLivingDetailsEntity.setRootAppNewData(JSON.toJSONString(ofsWareConfirmaData));//原始数据json
                    integrationTaskLivingDetailsEntity.setNewTransmitInfo(JSON.toJSONString(message));//返回结果
                    integrationTaskLivingDetailsEntity.setNewPushDate(new Date());
                    integrationTaskLivingDetailsEntity.setBusinessDate(generateBusinessDate);
                    integrationTaskLivingDetailsEntity.setRootAppPk(header.getId());
                    integrationTaskLivingDetailsEntity.setRootAppBill(header.getCode());
                    integrationTaskLivingDetailsEntity.setPluginId(getPluginId());
                    saveOrUpdateBusinessLogUtil.saveOrUpdate(integrationTaskLivingDetailsEntity);
                }
            }
        } else {
            logger.info("queryBasicArchives对应returnGoodHeaderDetailsDataDtoList.size为0");
        }
        return consignmachiningInSonDtoArrayList;
    }

    /**
     * 查询委外订单表头对象
     *
     * @author liuyang
     */
//    private ScOrderEntity queryScOrder(String ofsPoOrderId) {
//        Assert.notNull(ofsPoOrderId, "ofsPoOrderId不能为空!");
//        Assert.state(!"".equals(ofsPoOrderId.trim()), "ofsPoOrderId不能为空");
//
//        ScOrderEntity scOrderEntity = new ScOrderEntity();
//        scOrderEntity.setVdef20(ofsPoOrderId);
//        scOrderEntity.setDr("0");
//        scOrderEntity.setDataSourceCode("lets_u8c");
//        List<ScOrderEntity> scOrderEntityList = iScOrderDao.query(scOrderEntity);
//        if (scOrderEntityList == null || scOrderEntityList.size() == 0) {
//            Assert.state(false, "根据O采购订单主键:{}(u委外订单自定义项20存储O采购订单主键)，无法匹配到U8C委外订单", ofsPoOrderId);
//        } else if (scOrderEntityList.size() >= 2) {
//            Assert.state(false, "根据O采购订单主键:{}(u委外订单自定义项20存储O采购订单主键)，匹配到{}个U8C委外订单", ofsPoOrderId, scOrderEntityList.size());
//        }
//        return scOrderEntityList.get(0);
//    }

    /**
     * 2024年8月25日 14:11:31 查询U8C委外订单明细
     *
     * @param corderid 委外订单表头主键
     * @author liuyang
     */
//    private List<ScOrderBEntity> queryScOrderDetail(String corderid) throws Exception {
//        Assert.notNull(corderid, "委外订单表头主键不能为空！");
//        Assert.state(!"".equals(corderid), "委外订单表头主键不能为空！");
//
//        ScOrderBEntity scOrderBEntity = new ScOrderBEntity();
//        scOrderBEntity.setCorderid(corderid);
//        scOrderBEntity.setDr("0");
//        scOrderBEntity.setDataSourceCode("lets_u8c");
//        List<ScOrderBEntity> scOrderBEntityList = iScOrderBDao.query(scOrderBEntity);
//        if (scOrderBEntityList == null || scOrderBEntityList.size() == 0) {
//            //2024年8月25日 14:16:31 这个验证非常要必要，单据如果没有表体没有行，是很可疑的
//            Assert.state(false, "根据U8C委外订单主键({})无法查询到委外订单表体明细行！", corderid);
//        }
//        return scOrderBEntityList;
//    }

    /**
     * 2024年8月6日 10:59:03 查询U8C业务流程
     *
     * @author liuyang
     */
    private BdBusitypeEntity u8cOperationFlow() throws Exception {
//        Assert.notNull(pkCorp, "公司主键不能为空！");
//        Assert.state(!"".equals(pkCorp.trim()), "公司主键不能为空！");
        //查询业务流程
        //2024年8月6日 11:33:07 具体的业务流程名称，还需要实施提供
        String processName = "委外加工";
        BdBusitypeEntity bdBusitypeEntity = queryBdBusitypeUtil.queryBdBusitype(processName);
        Assert.notNull(bdBusitypeEntity, "根据业务流程名称({})没有查询到业务流程", processName);
        return bdBusitypeEntity;
    }

    /**
     * 2024年8月20日 15:46:10
     * 生成业务日期，以发货时间作为业务日期
     *
     * @author liuyang
     */
    private String createGenerateBusinessDate(PurchaseReceiptConfirmationSonDto purchaseReceiptConfirmationSonDto) {
        if (purchaseReceiptConfirmationSonDto != null && purchaseReceiptConfirmationSonDto.getReceivedAt() != null) {
            String receivedAt = purchaseReceiptConfirmationSonDto.getReceivedAt();

            String businessFormat = null;
            try {
                Date dbill = DateUtil.parse(receivedAt);
                businessFormat = DateUtil.format(dbill, "yyyy-MM-dd");
            } catch (Exception e) {
                logger.error("业务日期生成失败", e);
            }
            return businessFormat;
        } else {
            logger.error("生成业务日期失败，或者receivedAt为空! json：{}", JSON.toJSON(purchaseReceiptConfirmationSonDto));
            Assert.state(false, "生成业务日期失败，或者receivedAt为空! json：{}", JSON.toJSON(purchaseReceiptConfirmationSonDto));
            return null;
        }
    }

    /**
     * 2024年8月20日 15:46:10
     * 取 OFS采购入库确认单的入库时间
     * 生成业务日期，以发货时间作为业务日期
     *
     * @author liuyang
     */
    private String createGenerateBusinessDateV2(OFSReceiptConfirmSearchResponse.Header header) {
        if (header != null && header.getReceivedAt() != null) {
            String receivedAt = header.getReceivedAt();
            String businessFormat = null;
            try {
                Date dbill = DateUtil.parse(receivedAt);
                businessFormat = DateUtil.format(dbill, "yyyy-MM-dd");
            } catch (Exception e) {
                logger.error("业务日期生成失败", e);
            }
            return businessFormat;
        } else {
            logger.error("receivedAt为空! json：{}", JSON.toJSON(header));
            Assert.state(false, "receivedAt为空! json：{}", JSON.toJSON(header));
            return null;
        }
    }

    /**
     * 匹配U8C采购订单明细行
     * 通过O采购入库单明细行，得到采购订单明细行，最后查找到U8C采购订单明细行
     *
     * @param stockinB           OFS采购入库单明细行
     * @param ofsPoOrderData     OFS采购订单
     * @param scOrderBEntityList U8C委外订单明细行
     * @author liuyang
     */
//    private ScOrderBEntity findOfsPoOrderDetailAndU8cPoOrderDetail(StockinOrderSearchResponse.StockinOrder.StockinB stockinB, OfsPoOrderData ofsPoOrderData, List<ScOrderBEntity> scOrderBEntityList) {
//        Assert.notNull(stockinB, "stockinB不能为空");
//        Assert.notNull(ofsPoOrderData, "ofsPoOrderData不能为空");
//        Assert.notNull(scOrderBEntityList, "scOrderBEntityList删除不能为空");
//
////        OfsPoOrderHeader header = ofsPoOrderData.getHeader();
//        List<OfsPoOrderDetails> details = ofsPoOrderData.getDetails();
//
//        //查找采购订单明细行
//        OfsPoOrderDetails tarGetOfsPoOrderDetails = null;
//        for (int i = 0; i < details.size(); i++) {
//            OfsPoOrderDetails ofsPoOrderDetails = details.get(i);
//            if (ofsPoOrderDetails.getId().equals(stockinB.getRefOrderDetailId())) {
//                tarGetOfsPoOrderDetails = ofsPoOrderDetails;
//            }
//        }
//        Assert.notNull(tarGetOfsPoOrderDetails, "无法匹配到采购订单明细行！ 目标主键：{}", stockinB.getRefOrderDetailId());
//
//        //根据采购订单明细行主键，查找U8C采购订单明细行
//        for (int i = 0; i < scOrderBEntityList.size(); i++) {
//            ScOrderBEntity scOrderBEntity = scOrderBEntityList.get(i);
//            Assert.notNull(scOrderBEntity.getVdef20(), "委外订单明细行v20不能为空(明细行v20存储O采购订单明细行主键)！");
//            Assert.state(!"".equals(scOrderBEntity.getVdef20()), "委外订单明细行v20不能为空(明细行v20存储O采购订单明细行主键)！");
//            if (tarGetOfsPoOrderDetails.getId().equals(scOrderBEntity.getVdef20())) {
//                return scOrderBEntity;
//            }
//        }
//        Assert.state(false, "通过U8C采购订单明细行主键层层匹配到U8C采购明细失败！");
//        return null;
//    }

    /**
     * 匹配 U8C 采购入库单明细行
     * 通过 O 采购入库确认单明细行，匹配 O 采购入库单明细行
     * 通过 O 采购入库单明细行，得到采购订单明细行，最后查找到U8C采购订单明细行
     *
     * @param stockinB     OFS采购入库确认单明细行
     * @param stockinOrder OFS采购入库单
     * @author liuyang
     */
    private StockinOrderSearchResponse.StockinOrder.StockinB findOfsPoOrderInDetailRowTraget(OFSReceiptConfirmSearchResponse.Detail stockinB, StockinOrderSearchResponse.StockinOrder stockinOrder) {
        Assert.notNull(stockinB, "stockinB不能为空");
        Assert.notNull(stockinOrder, "stockinOrder不能为空");

        List<StockinOrderSearchResponse.StockinOrder.StockinB> details = stockinOrder.getDetails();
        for (int i = 0; i < details.size(); i++) {
            StockinOrderSearchResponse.StockinOrder.StockinB stockinB1 = details.get(i);
            if (stockinB1.getId().equals(stockinB.getRefOrderDetailId())) {
                return stockinB1;
            }
        }
        Assert.state(false, "通过OFS采购入库确认单，无法匹配到OFS采购入库单明细行！ 目标主键：{}", stockinB.getRefOrderDetailId());
        return null;
    }

    /**
     * 匹配 U8C 采购订单明细行，主要是为了取金额
     * 通过 O 采购入库单明细行，得到采购订单明细行，最后查找到U8C采购订单明细行,主要是为了取金额
     *
     * @param stockinB       OFS采购入库单明细行
     * @param ofsPoOrderData OFS采购订单
     * @author liuyang
     */
    private OfsPoOrderDetails findOfsPoOrderDetailRowTraget(StockinOrderSearchResponse.StockinOrder.StockinB stockinB, OfsPoOrderData ofsPoOrderData) {
        Assert.notNull(stockinB, "stockinB不能为空");
        Assert.notNull(ofsPoOrderData, "stockinOrder不能为空");

        List<OfsPoOrderDetails> details = ofsPoOrderData.getDetails();
        for (int i = 0; i < details.size(); i++) {
            OfsPoOrderDetails ofsPoOrderDetails = details.get(i);
            if (ofsPoOrderDetails.getId().equals(stockinB.getRefOrderDetailId())) {
                return ofsPoOrderDetails;
            }
        }
        Assert.state(false, "通过OFS采购入库单，无法匹配到OFS采购订单明细行（为了取采购订单明细行金额）！ 目标主键：{}", stockinB.getRefOrderDetailId());
        return null;
    }

    /**
     * 查询存货管理档案
     *
     * @param pkCorp   发货公司主键
     * @param stockinB 采购入库单明细行
     * @author liuyang
     */
    private BdInvmandocEntity queryInventoryMan(OFSReceiptConfirmSearchResponse.Detail stockinB, String pkCorp) throws Exception {
        Assert.notNull(stockinB, "sonDetailsDto不能为空");
        Assert.notNull(stockinB.getSkuCode(), "O存货商家编码不能为空");
        Assert.notNull(pkCorp, "发货公司主键不能为空");

        BdInvmandocEntity bdInvmandocEntity = new BdInvmandocEntity();
        bdInvmandocEntity.setInvcode(stockinB.getSkuCode());
        bdInvmandocEntity.setPkCorp(pkCorp);
        List<BdInvmandocEntity> bdInvmandocEntity2 = iBdInvmandocDao.queryBdInvmandocByInvcodeList(bdInvmandocEntity);
        if (bdInvmandocEntity2.size() == 0) {
            Assert.state(false, "根据O商家编码：{} U8C发货公司主键：{}没有查询到存货管理档案", stockinB.getSkuCode(), pkCorp);
        } else if (bdInvmandocEntity2.size() >= 2) {
            Assert.state(false, "根据O商家编码：{} U8C发货公司主键：{}没有查询到存货管理档案", stockinB.getSkuCode(), pkCorp);
        }
        return bdInvmandocEntity2.get(0);
    }

    /**
     * 2024年8月25日 15:12:22
     * 推送U8C委外入库单
     *
     * @param param 原数据json
     * @author liuyang
     */
//    public ScOrderResultDto sendU8cScorder(String param) throws Exception {
//        long startLong = System.currentTimeMillis();
//        logger.info("U8C委外订单推送开始，推送参数：" + param + ",U8C_URL:" + OverallConstant.getOverAllValue("u8c_url"));
//        String apiCode = "8000370037";
//        String result = HttpRequest.post(OverallConstant.getOverAllValue("u8c_url")).header("appId", OverallConstant.getOverAllValue("appId"))//头信息，多个头信息多次调用此方法即可
//                .header("usercode", OverallConstant.getOverAllValue("u8cApiUsercodePK")).header("password", OverallConstant.getOverAllValue("u8cApiPasswordPK")).header("system", OverallConstant.getOverAllValue("u8cApiSystemPK")).header("trantype", OverallConstant.getOverAllValue("u8cApiTrantypePK")).header("apiCode", apiCode)//头信息，多个头信息多次调用此方法即可
//                .header("publicKey", OverallConstant.getOverAllValue("publicKey"))//头信息，多个头信息多次调用此方法即可
//                .header("secretKey", OverallConstant.getOverAllValue("secretKey"))//头信息，多个头信息多次调用此方法即可
//                .body(param)//表单内容
//                .timeout(600000)//超时，毫秒
//                .execute().body();
//        logger.info("U8C委外订单推送结束，返回参数：" + result);
//        long endLong = System.currentTimeMillis();
//        logger.info("U8C委外订单推送耗时：" + (endLong - startLong));
//
//        if (result == null) {
//            result = "";
//        }
//
//        JSONObject jsonObject = JSON.parseObject(result);
//        result = String.valueOf(jsonObject.get("attribute"));
//
//        boolean isSuccess = false;
//        ScOrderResultDto scOrderResultDto = null;
//        if (result != null && !"".equals(result)) {
//            ReusltStrDto reusltStrDto = JSON.parseObject(result, ReusltStrDto.class);
//            if ("success".equals(reusltStrDto.getStatus())) {
//                scOrderResultDto = resultDataHandle(reusltStrDto.getData());
//                isSuccess = true;
//            }
//        }
//        if (!isSuccess) {
//            Assert.state(false, "O采购入库业务(委外入库)推送U8C委外订单失败 接口返回结果：{}", result);
//        }
//        return scOrderResultDto;
//    }

    /**
     * 返回结果解析处理，在确认了success后调用
     *
     * @author liuyang
     */
    private IcGeneralHResultDto resultDataHandle(String resultData) {
        try {
            if (resultData != null && !"".equals(resultData)) {
                if (resultData.contains("[")) {
                    resultData = resultData.substring(1, resultData.length() - 1);
                }
                return JSON.parseObject(resultData, IcGeneralHResultDto.class);
            }
        } catch (Exception e) {
            logger.error("resultDataHandle方法解析返回参数失败的错误", e);
            //如果解析失败，记录原因，但是不能影响结果的记录
        }
        return null;
    }

    /**
     * 查询BOM版本，版本最新的在返回的结果集的最前面
     *
     * @param gcbm   工厂主键
     * @param wlbmid 物料编码id，经过测试，这是存货基本档案主键
     */
//    private List<BdBomEntity> getBomVersion(String gcbm, String wlbmid, String vbillCode) throws Exception {
//        Assert.notNull(gcbm, "gcbm工厂主键");
//        Assert.notNull(wlbmid, "wlbmid物料编码id不能为空");
//
//        BdBomEntity bdBomEntity = new BdBomEntity();
//        bdBomEntity.setDataSourceCode("lets_u8c");
//        bdBomEntity.setWlbmid(wlbmid);
//        bdBomEntity.setGcbm(gcbm);
//        List<BdBomEntity> bdBomEntityList = iBdBomDao.query(bdBomEntity);
//        //2024年9月2日 15:08:30 等待妮姐确认，如果对应的存货没有BOM，是否抛出异常
//        //2024年9月2日 15:14:18 已经和妮姐确认，如果存货对应的BOM不存在，则抛出异常
//        if (bdBomEntityList == null || bdBomEntityList.size() == 0) {
//            Assert.state(false, "存货：{}对应的BOM不存在(该存货需要维护BOM)!", vbillCode);
//        }
//        return bdBomEntityList;
//    }

    /**
     * 2024年8月7日 10:25:29
     * 查询基础档案，根据公司、管理档案主键查询
     *
     * @author liuyang
     */
    private BdInvbasdocEntity queryStockBasicArchives(String pkInvmandoc, String pkCorp) throws Exception {
        Assert.notNull(pkInvmandoc, "存货管理档案不能为空");
        Assert.notNull(pkCorp, "公司档案不能为空");

        BdInvbasdocEntity bdInvbasdocEntity = new BdInvbasdocEntity();
        bdInvbasdocEntity.setPk_invmandoc(pkInvmandoc);
        bdInvbasdocEntity.setPk_corp(pkCorp);
        List<BdInvbasdocEntity> bdInvbasdocEntity2 = iBdInvbasdocDao.queryBdInvbasdocByPkInvmandocV2(bdInvbasdocEntity);
        if (bdInvbasdocEntity2.size() == 0) {
            Assert.state(false, "根据存货管理档案主键{}和公司档案主键{}没有查询到U8C基础档案", pkInvmandoc, pkCorp);
        } else if (bdInvbasdocEntity2.size() >= 2) {
            Assert.state(false, "根据存货管理档案主键{}和公司档案主键{}没有查询到U8C基础档案", pkInvmandoc, pkCorp);
        }
        return bdInvbasdocEntity2.get(0);
    }

    /**
     * 分割日期之后，再进行推送
     *
     * @author liuyang
     */
    private void splitDateAndPush(String param) throws Exception {
        try {
            Assert.notNull(param, "param不能为空!");
            Assert.state(!"".equals(param), "param不能为空！");

            if (param.contains("*")) {
                String[] params = param.split("\\*");
                if (params.length > 0) {
                    for (int i = 0; i < params.length; i++) {
                        String indexStr = params[i];
                        String[] split = indexStr.split(",");
                        startImplementByTime(split[0], split[1]);
                    }
                }
            } else {
                String[] split = param.split(",");
                startImplementByTime(split[0], split[1]);
            }
        } catch (Exception e) {
            logger.error("记录splitDateAndPush方法抛出的异常", e);
        }
    }

    /**
     * 打印OFS单号
     *
     * @param returnGoodHeaderDetailsDataDtoArrayList 采购入库确认单返回结果
     * @param startTime                               开始时间
     * @param endTime                                 结束时间
     * @author liuyang
     */
    private void printOfsDocCode(List<OFSReceiptConfirmSearchResponse.Order> returnGoodHeaderDetailsDataDtoArrayList, String startTime, String endTime) {
        Assert.notNull(returnGoodHeaderDetailsDataDtoArrayList, "returnGoodHeaderDetailsDataDtoArrayList不能为空");
        Assert.notNull(startTime, "startTime不能为空");
        Assert.notNull(endTime, "endTime不能为空");

        try {
            StringBuffer stringBuffer = new StringBuffer();
            stringBuffer.append("插件：");
            stringBuffer.append(getPluginName());
            stringBuffer.append(" ");

            stringBuffer.append("开始时间：");
            stringBuffer.append(startTime);
            stringBuffer.append(" ");

            stringBuffer.append("结束时间：");
            stringBuffer.append(endTime);
            stringBuffer.append(" ");

            stringBuffer.append("获取到的单号：");
            String codes = returnGoodHeaderDetailsDataDtoArrayList.stream().map(returnGoodHeaderDetailsDataDto -> returnGoodHeaderDetailsDataDto.getHeader().getCode()).collect(Collectors.joining(","));
            stringBuffer.append(codes);
            logger.info(stringBuffer.toString());
        } catch (Exception e) {
            logger.error("插件:{}，打印拉取到的单据号出错", getPluginName(), e);
        }
    }

    /**
     * 打印OFS单号
     *
     * @param returnGoodHeaderDetailsDataDtoArrayList 采购入库确认单返回结果
     * @param code                                    入库确认单-单号
     * @author liuyang
     */
    private void printOfsDocCode(List<OFSReceiptConfirmSearchResponse.Order> returnGoodHeaderDetailsDataDtoArrayList, String code) {
        Assert.notNull(returnGoodHeaderDetailsDataDtoArrayList, "returnGoodHeaderDetailsDataDtoArrayList不能为空");
        Assert.notNull(code, "code不能为空");

        try {
            StringBuffer stringBuffer = new StringBuffer();
            stringBuffer.append("插件：");
            stringBuffer.append(getPluginName());
            stringBuffer.append(" ");

            stringBuffer.append("单据号：");
            stringBuffer.append(code);
            stringBuffer.append(" ");

            stringBuffer.append("获取到的单号：");
            String codes = returnGoodHeaderDetailsDataDtoArrayList.stream().map(returnGoodHeaderDetailsDataDto -> returnGoodHeaderDetailsDataDto.getHeader().getCode()).collect(Collectors.joining(","));
            stringBuffer.append(codes);
            logger.info(stringBuffer.toString());
        } catch (Exception e) {
            logger.error("插件:{}，打印拉取到的单据号出错", getPluginName(), e);
        }
    }

    /**
     * 查询委外订单表头对象
     *
     * @param ofsPoInCode OFS采购入库单号
     * @author liuyang
     */
    private ScOrderEntity queryScOrder(String ofsPoInCode) {
        Assert.notNull(ofsPoInCode, "OFS 采购入库单号不能为空!");
        Assert.state(!"".equals(ofsPoInCode.trim()), "OFS 采购入库单号不能为空!");

        ScOrderEntity scOrderEntity = new ScOrderEntity();
        scOrderEntity.setVdef17(ProfilesActiveConstant.sourceSystem1);
        scOrderEntity.setVdef19(ofsPoInCode);
        scOrderEntity.setDr("0");
        scOrderEntity.setDataSourceCode("lets_u8c");
        List<ScOrderEntity> scOrderEntityList = iScOrderDao.query(scOrderEntity);
        if (scOrderEntityList == null || scOrderEntityList.size() == 0) {
            Assert.state(false, "根据O采购入库单编码:{}，无法匹配到U8C委外订单", ofsPoInCode);
        } else if (scOrderEntityList.size() >= 2) {
            Assert.state(false, "根据O采购入库单编码:{}，无法匹配到U8C委外订单，匹配到{}个U8C委外订单", ofsPoInCode, scOrderEntityList.size());
        }
        return scOrderEntityList.get(0);
    }

    /**
     * 匹配U8C采购订单明细行
     * 通过O采购入库确认单明细行，匹配 O 采购入库单明细行，再匹配到U8C 委外订单明细行
     *
     * @param ofsWareConfirmaDetails OFS采购入库确认单明细行
     * @param stockinOrder           O采购入库单明细
     * @param ofsPoOrderData         O采购订单明细行
     * @param scOrderBEntityList     O委外订单
     * @author liuyang
     */
    private ScOrderBEntity findOfsPoOrderDetailAndU8cPoOrderDetail(OFSReceiptConfirmSearchResponse.Detail ofsWareConfirmaDetails, StockinOrderSearchResponse.StockinOrder stockinOrder, OfsPoOrderData ofsPoOrderData, List<ScOrderBEntity> scOrderBEntityList) throws Exception {
        Assert.notNull(ofsWareConfirmaDetails, "ofsWareConfirmaDetails不能为空！");
        Assert.notNull(stockinOrder, "stockinOrder不能为空！");
        Assert.notNull(scOrderBEntityList, "scOrderBEntityList不能为空！");
        Assert.notNull(ofsPoOrderData, "ofsPoOrderData不能为空！");

        //逻辑一
        //通过O采购入库确认单，匹配O采购入库单明细行
        //因为O确认单明细行没有返回O采购入库单明细行主键，无法关联
//        StockinOrderSearchResponse.StockinOrder.StockinB targetPurchaseReturnOrderDetails = null;
//        List<StockinOrderSearchResponse.StockinOrder.StockinB> details = stockinOrder.getDetails();
//        for (int i = 0; i < details.size(); i++) {
//            StockinOrderSearchResponse.StockinOrder.StockinB stockinB = details.get(i);
//            if (ofsWareConfirmaDetails.getRefOrderDetailId().equals(stockinB.getId())) {
//                targetPurchaseReturnOrderDetails = stockinB;
//            }
//        }
//        Assert.notNull(targetPurchaseReturnOrderDetails, "通过O入库确认单refOrderDetailId({})，无法匹配O采购入库单明细行！", ofsWareConfirmaDetails.getRefOrderDetailId());

        //逻辑二
        //通过O确认单明细行关联O采购订单明细行
        List<OfsPoOrderDetails> details1 = ofsPoOrderData.getDetails();
        OfsPoOrderDetails targetOfsPoOrderDetails = null;
//        List<StockinOrderSearchResponse.StockinOrder.StockinB> details = stockinOrder.getDetails();
        for (int i = 0; i < details1.size(); i++) {
            OfsPoOrderDetails ofsPoOrderDetails = details1.get(i);
            if (ofsWareConfirmaDetails.getRefOrderDetailId().equals(ofsPoOrderDetails.getId())) {
                targetOfsPoOrderDetails = ofsPoOrderDetails;
            }
        }
        Assert.notNull(targetOfsPoOrderDetails, "通过O入库确认单refOrderDetailId({})，无法匹配O订单明细行！", ofsWareConfirmaDetails.getRefOrderDetailId());

        //通过O采购订单单明细行，匹配U8C委外订单明细行
        for (int i = 0; i < scOrderBEntityList.size(); i++) {
            ScOrderBEntity scOrderBEntity = scOrderBEntityList.get(i);
            if (targetOfsPoOrderDetails.getId().equals(scOrderBEntity.getVdef20())) {
                return scOrderBEntity;
            }
        }
        Assert.state(false, "O采购订单明细行主键：{},无法匹配到U8C委外订单明细行！", targetOfsPoOrderDetails.getId());
        return null;
    }

    /**
     * 通过O确认单明细行关联O采购订单明细行
     *
     * @param ofsWareConfirmaDetails OFS采购入库确认单明细行
     * @param ofsPoOrderData         O采购订单明细行
     * @author liuyang
     */
    private OfsPoOrderDetails findOfsPoOrderDetailAndU8cPoOrderDetail(OFSReceiptConfirmSearchResponse.Detail ofsWareConfirmaDetails, OfsPoOrderData ofsPoOrderData) throws Exception {
        Assert.notNull(ofsWareConfirmaDetails, "ofsWareConfirmaDetails不能为空！");
        Assert.notNull(ofsPoOrderData, "ofsPoOrderData不能为空！");

        //通过O确认单明细行关联O采购订单明细行
        List<OfsPoOrderDetails> details1 = ofsPoOrderData.getDetails();
        OfsPoOrderDetails targetOfsPoOrderDetails = null;
        for (int i = 0; i < details1.size(); i++) {
            OfsPoOrderDetails ofsPoOrderDetails = details1.get(i);
            if (ofsWareConfirmaDetails.getRefOrderDetailId().equals(ofsPoOrderDetails.getId())) {
                targetOfsPoOrderDetails = ofsPoOrderDetails;
            }
        }
        Assert.notNull(targetOfsPoOrderDetails, "通过O入库确认单refOrderDetailId({})，无法匹配O订单明细行！", ofsWareConfirmaDetails.getRefOrderDetailId());
        return targetOfsPoOrderDetails;
    }

    /**
     * 2024年8月25日 14:11:31 查询U8C委外订单明细
     *
     * @param corderid 委外订单表头主键
     * @author liuyang
     */
    private List<ScOrderBEntity> queryScOrderDetail(String corderid) throws Exception {
        Assert.notNull(corderid, "委外订单表头主键不能为空！");
        Assert.state(!"".equals(corderid), "委外订单表头主键不能为空！");

        ScOrderBEntity scOrderBEntity = new ScOrderBEntity();
        scOrderBEntity.setCorderid(corderid);
        scOrderBEntity.setDr("0");
        scOrderBEntity.setDataSourceCode("lets_u8c");
        List<ScOrderBEntity> scOrderBEntityList = iScOrderBDao.query(scOrderBEntity);
        if (scOrderBEntityList == null || scOrderBEntityList.size() == 0) {
            //2024年8月25日 14:16:31 这个验证非常要必要，单据如果没有表体没有行，是很可疑的
            Assert.state(false, "根据U8C委外订单主键({})无法查询到委外订单表体明细行！", corderid);
        }
        return scOrderBEntityList;
    }

    /**
     * 2024年8月30日 14:09:26
     * 推送U8C入库单
     *
     * @param param 原数据json
     * @author liuyang
     */
    public IcGeneralHResultDto sendU8cConsignmachiningIn(String param) throws Exception {
        long startLong = System.currentTimeMillis();
        logger.info("U8C委外入库推送开始，推送参数：" + param + ",U8C_URL:" + OverallConstant.getOverAllValue("u8c_url"));
        String apiCode = "8000370056";
        String result = HttpRequest.post(OverallConstant.getOverAllValue("u8c_url")).header("appId", OverallConstant.getOverAllValue("appId"))//头信息，多个头信息多次调用此方法即可
                .header("usercode", OverallConstant.getOverAllValue("u8cApiUsercodePK")).header("password", OverallConstant.getOverAllValue("u8cApiPasswordPK")).header("system", OverallConstant.getOverAllValue("u8cApiSystemPK")).header("trantype", OverallConstant.getOverAllValue("u8cApiTrantypePK")).header("apiCode", apiCode)//头信息，多个头信息多次调用此方法即可
                .header("publicKey", OverallConstant.getOverAllValue("publicKey"))//头信息，多个头信息多次调用此方法即可
                .header("secretKey", OverallConstant.getOverAllValue("secretKey"))//头信息，多个头信息多次调用此方法即可
                .body(param)//表单内容
                .timeout(600000)//超时，毫秒
                .execute().body();
        logger.info("U8C委外入库推送结束，返回参数：" + result);
        long endLong = System.currentTimeMillis();
        logger.info("U8C委外入库推送耗时：" + (endLong - startLong));

        if (result == null) {
            result = "";
        }

        JSONObject jsonObject = JSON.parseObject(result);
        result = String.valueOf(jsonObject.get("attribute"));

        boolean isSuccess = false;
        IcGeneralHResultDto icGeneralHResultDto = null;
        if (result != null && !"".equals(result)) {
            ReusltStrDto reusltStrDto = JSON.parseObject(result, ReusltStrDto.class);
            if ("success".equals(reusltStrDto.getStatus())) {
                icGeneralHResultDto = resultDataHandle(reusltStrDto.getData());
                isSuccess = true;
            }
        }
        if (!isSuccess) {
            Assert.state(false, "O采购入库业务推送U8C委外入库失败 接口返回结果：{}", result);
        }
        return icGeneralHResultDto;
    }

    /**
     * 批量查询O 采购入库单
     *
     * @author liuyang
     */
    private void queryBatchOfsPoOrderIn(List<OFSReceiptConfirmSearchResponse.Order> purchaseReceiptConfirmationSonDtoList) throws Exception {
        Set<String> codeSet = null;
        if (purchaseReceiptConfirmationSonDtoList != null && purchaseReceiptConfirmationSonDtoList.size() > 0) {
            codeSet = purchaseReceiptConfirmationSonDtoList.stream().filter(dto -> dto.getHeader() != null).filter(dto -> dto.getHeader().getCode() != null).map(dto -> dto.getHeader().getCode().replace("-confirm", "")).collect(Collectors.toSet());
        }
        if (codeSet != null && codeSet.size() > 0) {
            for (String element : codeSet) {
                List<StockinOrderSearchResponse.StockinOrder> returnGoodHeaderDetailsDataDtoArrayList = new ArrayList<>();
                QueryOfsSoSaleOutVo queryOfsSoSaleOutVo = new QueryOfsSoSaleOutVo();
                queryOfsSoSaleOutVo.setClientCode("LETS");
                queryOfsSoSaleOutVo.setPageNo(1L);
                queryOfsSoSaleOutVo.setPageSize(50L);
                queryOfsSoSaleOutVo.setCode(element);
                queryOfsSoSaleOutVo.setInternalInstructionType("PURCHASE");
                ofsStandardUtil.queryOfsReturnGoods(queryOfsSoSaleOutVo, returnGoodHeaderDetailsDataDtoArrayList, 1L, "ofs.receipt.search");
                if (returnGoodHeaderDetailsDataDtoArrayList.size() == 0) {
                    Assert.state(false, "根据采购入库单号：{}无法查询到数据", element);
                }
                //「O采购入库确认单」关联「O采购入库单」
                StockinOrderSearchResponse.StockinOrder stockinOrder = returnGoodHeaderDetailsDataDtoArrayList.get(0);
                for (OFSReceiptConfirmSearchResponse.Order order : purchaseReceiptConfirmationSonDtoList) {
                    OFSReceiptConfirmSearchResponse.Header header = order.getHeader();
                    if (header.getCode() != null) {
                        String replace = header.getCode().replace("-confirm", "");
                        if (element.equals(replace)) {
                            header.setOfsStockinOrder(stockinOrder);
                        }
                    }
                }
            }
        }
    }

    /**
     * 批量查询 OFS 采购订单
     *
     * @param purchaseReceiptConfirmationSonDtoList O采购入库确认单
     * @author liuyang
     */
    private void queryBatchOfsPoOrder(List<OFSReceiptConfirmSearchResponse.Order> purchaseReceiptConfirmationSonDtoList) throws Exception {
        Set<String> refOrderCodeSet = null;
        if (purchaseReceiptConfirmationSonDtoList != null && purchaseReceiptConfirmationSonDtoList.size() > 0) {
            List<StockinOrderSearchResponse.StockinOrder> stockinOrderList = new ArrayList<>();
            for (OFSReceiptConfirmSearchResponse.Order purchaseReceiptConfirmationSonDto : purchaseReceiptConfirmationSonDtoList) {
                OFSReceiptConfirmSearchResponse.Header header = purchaseReceiptConfirmationSonDto.getHeader();
                if (header != null && header.getOfsStockinOrder() != null) {
                    stockinOrderList.add(header.getOfsStockinOrder());
                }
            }
            if (stockinOrderList.size() > 0) {
                refOrderCodeSet = Optional.ofNullable(stockinOrderList).orElseGet(Collections::emptyList).stream().filter(Objects::nonNull).map(StockinOrderSearchResponse.StockinOrder::getHeader).filter(Objects::nonNull).map(StockinOrderSearchResponse.StockinOrder.StockinH::getRefOrderCode).filter(Objects::nonNull).collect(Collectors.toSet());
            }
        }

        if (refOrderCodeSet != null && refOrderCodeSet.size() > 0) {
            for (String refOrderCode : refOrderCodeSet) {
                OfsPoOrderData ofsPoOrderData = ofsStandardUtil.queryOfsPoOrder(refOrderCode);
                Assert.notNull(ofsPoOrderData, "无法查询到O采购订单 编码：{}", refOrderCode);

                for (OFSReceiptConfirmSearchResponse.Order purchaseReceiptConfirmationSonDto : purchaseReceiptConfirmationSonDtoList) {
                    if (purchaseReceiptConfirmationSonDto.getHeader() != null) {
                        OFSReceiptConfirmSearchResponse.Header header2 = purchaseReceiptConfirmationSonDto.getHeader();

                        //O采购入库单
                        Assert.notNull(header2.getOfsStockinOrder(), "O采购入库单不能为空");
                        StockinOrderSearchResponse.StockinOrder ofsStockinOrder = header2.getOfsStockinOrder();
                        StockinOrderSearchResponse.StockinOrder.StockinH header3 = ofsStockinOrder.getHeader();

                        if (header3.getRefOrderCode().equals(refOrderCode)) {
                            header2.setOfsOrderData(ofsPoOrderData);
                        }
                    }
                }
            }
        }
    }
}